/*
 * UNICADO - UNIversity Conceptual Aircraft Design and Optimization
 *
 * Copyright (C) 2025 UNICADO consortium
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * Description:
 * This file is part of UNICADO.
 */

#ifndef CALCULATEPOLAR_SRC_BWB_BWBCALCULATEPOLARCONFIG_H_
#define CALCULATEPOLAR_SRC_BWB_BWBCALCULATEPOLARCONFIG_H_

#include "../flightCondition.h"

#include <aixml/endnode.h>

#include <map>
#include <set>
#include <string>
#include <vector>

/** \brief Settings class for calculatePolar. Initializes tool with settings set in the respective config file.
 *  Inherits from the settings class (located in libraries and in unicadoRuntimeLibs)
 */
class bwbCalculatePolarConfig{
 public:
    /* !!! WARNING: All EndnodeReadOnly Values are converted automatically to SI-Units/RADIAN when read in */
    EndnodeReadOnly<std::string>pathToLiftingLine;
    // general lifting line settings
    EndnodeReadOnly<std::string> CLModeAirfoilSelection;    /**< Switch which sets if lift coefficient at ICA, at TOC or at average cruise is chosen for airfoil selection */
    EndnodeReadOnly<double> stepWidthCL;
    // fligt condition parameters
    bool setFlightConditionsMode;
    // lifting line pitching moment parameters
    EndnodeReadOnly<double> delta_CM0;
    EndnodeReadOnly<double> delta_dCMdCL;
    // lifting line for TAW induced drag calibraitons
    EndnodeReadOnly<double> indDragCtCorrForCalibration;
    EndnodeReadOnly<double> factorIndDragCleanPolar;
    // visc drag raymer
    EndnodeReadOnly<bool> manualTransition;
    EndnodeReadOnly<double> manualTransitionWing;
    EndnodeReadOnly<double> manualTransitionFin;
    // visc drag raymer calibration parameters
    EndnodeReadOnly<bool> doViscDragCalibration;
    EndnodeReadOnly<double> CDvisc_CDsum;
    EndnodeReadOnly<double> CDvisc_CLfact;
    EndnodeReadOnly<double> CDvisc_CLexp;
    EndnodeReadOnly<double> CDvisc_CDsum_lowMa;
    EndnodeReadOnly<double> CDvisc_CLfact_lowMa;
    EndnodeReadOnly<double> CDvisc_CLexp_lowMa;
    EndnodeReadOnly<bool> doViscousDragRaymerDragCountCorrection;
    EndnodeReadOnly<double> deltaViscousDragCountWing;
    EndnodeReadOnly<double> deltaViscousDragCountVTP;
    EndnodeReadOnly<double> deltaViscousDragCountNacelles;
    EndnodeReadOnly<double> deltaViscousDragCountMisc;
    // wave drag mason
    EndnodeReadOnly<bool> useCostomReferenceAngleForSweep;
    EndnodeReadOnly<bool> doWaveDragMasonCalibration;
    EndnodeReadOnly<bool> doWaveDragMasonDragCountCorrection;
    EndnodeReadOnly<int> numberWingStrips;
    EndnodeReadOnly<double> Ka;
    EndnodeReadOnly<double> maximumSegmentSweep;
    EndnodeReadOnly<double> customSweepPosition;
    EndnodeReadOnly<double> CDwave_CLFact;
    EndnodeReadOnly<double> CDwave_CLExp;
    EndnodeReadOnly<double> deltaWaveDragCountWing;
    //semi empirical high lift adaptions
    EndnodeReadOnly<double> deltaTotalDragHL;
    EndnodeReadOnly<double> factorDragHL;

    explicit bwbCalculatePolarConfig(const node& config);
    virtual ~bwbCalculatePolarConfig() {}
    // explicit calculatePolarSettings(const node& configXML);
    /** \brief  Virtual destructor of calculatePolarSettings
     */
    // virtual ~calculatePolarSettings();
    /* Member Variables */
    std::string liftingLineFolderPath; /**< Path to LiftingLine folder */
    bool deleteLLfolder; /**< Switch which sets if liftingLine folder in project folder is deleted after execution */
    std::string aeroDatabaseFolderPath; /**< Path to AeroDatabase folder */
    bool getAlternativeDragComponentsFromDB; /**< Switch if also the friction and pressure drag should be read from DB and plotted */
    std::string stabtoolFolderPath; /**< Path to Stabtool folder */

    std::vector<std::string> CGPositions;  /**< vector of CG positions, order: design, fwd, aft*/

    /* Polar Configuration */
    /** \brief  Class which contains configurations for the polar calculations
     */
    class polarConfig {
     public:
        double stepWidthCL; /**< Step width of lift coefficient (CL) values within the polar */
        double stepWidthCLforCalculation;/**< Step width of lift coefficient (CL) values within the polar for calculations \note variable within database method */
    };
    polarConfig Polar; /**< Initialization of a polarConfig object */
    uint8_t plottingIncrement; /**< variable within database method */

    /* Flight Conditions */
    bool adaptMachDistribution; /**< Switch to either use the list of flight conditions from the config file or to distribute them reasonably with respect to design mach number */
    double cruiseMach; /**< Cruise mach number [-] */
    size_t cruiseMach_ID; /**< ID of the mach number's flight condition [-] */
    double designAltitude; /**< Design altitude [ft] */
    bool useAverageCruiseAltitude; /**< Switch to either use the given design altitude or to calculate the average from the flown cruise steps set in aricraft xml */
    int numberOfCruiseSteps; /**< Number of cruise steps in design mission*/
    int numberOfFlightConditions; /**< Number of flight conditions [-] */
    std::vector <double> lowSpeed_Mach; /**< Vector of low speed mach numbers. Only used if adaptMachDistribution is set to true */
    std::vector <double> highSpeed_Mach_Deltas; /**< Vector of high speed mach number deviations. Used to distribute mach numbers around cruise mach number */
    std::vector <double> FreestreamMachNumbers; /**< Vector of Freestream Mach numbers */
    std::vector<flightCondition> flightConditions; /**< Vector of flightCondition objects representing the mission data */
    std::vector<double> machNumbers;  /**< Vector with mach numbers corrsponding to their mach ID */

 private:
    /* Member functions */
    /** \brief Function reads polar settings from configuration XML and stores them in polarConfig object
     *  \param &polarNode aixml node that contains polar configuration information from corresponding XML file
     */
    void readPolarSettings(const node& polarNode);
    /** \brief Function reads flight condition settings from configuration XML and stores them in class variables
     *  \param &config aixml node that contains the config root
     *  \param ID an index value of the current flight condition
     */
    void readFlightConditions(const node& config, size_t ID);
    /** \brief Function reads viscous drag settings from configuration XML and stores them in class variables
     *  \param &viscousDragNode aixml node that contains flight condition information from corresponding XML file
     */
    void readViscousDragSettings(const node& viscousDragNode);
    /** \brief Function reads wave drag settings from configuration XML and stores them in class variables
     *  \param &viscousDragNode aixml node that contains flight condition information from corresponding XML file
     */
    void readWaveDragSettings(const node& viscousWaveNode);
};

#endif  // CALCULATEPOLAR_SRC_BWB_BWBCALCULATEPOLARCONFIG_H_
