# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

# Import standard modules.
import numpy as np
from matplotlib import pyplot as plt


def generate_landing_gear_plots(dict_ac_exchange, landing_gear_description, runtime_output):
    """ Function to generate all necessary outputs to plot side and top view of landing gear design.

    :param dict dict_ac_exchange: Dict containing parameter and according values from aircraft exchange file
    :param dict landing_gear_description: Dictionary contains all global constants and conversion factors
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return: dict dict_landing_gear_output: Dictionary contains all outputs for plot generation of landing gear design
    """
    if not dict_ac_exchange['configuration_type'] == 'blended_wing_body':
        
        ''' initialize local parameter '''
        dict_landing_gear_output = {}
        convert_inch_to_meter = 0.0254

        ''' generate global reference point '''
        dict_landing_gear_output['global_reference_point'] = {'x': 0.0, 'y': 0.0, 'z': 0.0},

        ''' generate output data for front view of landing gear design '''
        max_fuselage_section_upper_height = round(max(dict_ac_exchange['fuselage_section_upper_height'].values()), 4)
        max_fuselage_section_lower_height = -round(max(dict_ac_exchange['fuselage_section_lower_height'].values()), 4)
        semi_fuselage_section_width = round(max(dict_ac_exchange['fuselage_section_width'].values())/2, 4)

        # Compute the value of y for the upper and lower boundaries of the hyperellipse
        dict_landing_gear_output['width_cross_section_points'] = np.linspace(-semi_fuselage_section_width,
                                                                            semi_fuselage_section_width,
                                                                            500)
        dict_landing_gear_output['upper_cross_section_points'] = \
            (max_fuselage_section_upper_height * np.sqrt(1 - (pow(dict_landing_gear_output['width_cross_section_points'], 2)
                                                            / pow(semi_fuselage_section_width, 2))))
        dict_landing_gear_output['lower_cross_section_points'] = \
            (max_fuselage_section_lower_height * np.sqrt(1 - (pow(dict_landing_gear_output['width_cross_section_points'], 2)
                                                            / pow(semi_fuselage_section_width, 2))))

        # Generate a list of lists of all tires are visible in the front view.
        tire_plot_list = []
        strut_positions = []

        # Add nose gear tires to tire plot list.
        if landing_gear_description['nose_gear_description']['nose_gear_arrangement']['number_of_nose_gear_struts'] == 1:
            max_section_width = float(next((value['max_section_width_in_inch'] for value in
                                            landing_gear_description['nose_gear_description'][
                                                'nose_gear_tires'].values()
                                            if 'max_section_width_in_inch' in value), None))
            max_tire_diameter = float(next((value['max_outside_diameter_in_inch'] for value in
                                            landing_gear_description['nose_gear_description'][
                                                'nose_gear_tires'].values()
                                            if 'max_outside_diameter_in_inch' in value), None))
            i = 0
            while i < landing_gear_description['nose_gear_description']['nose_gear_arrangement']['number_of_nose_gear_tires_per_strut']:  # noPep8 e501
                tire_plot_list.append(
                    [round(landing_gear_description['landing_gear_positions']['nose_gear_strut_y_position']
                        + landing_gear_description['landing_gear_cog']['nose_gear_cog'][
                            'tire_positions_per_strut']['y'][i]
                        - (max_section_width / 2 * convert_inch_to_meter), 4),
                    round(landing_gear_description['landing_gear_positions']['nose_gear_z_position']
                        + landing_gear_description['landing_gear_cog']['nose_gear_cog'][
                            'tire_positions_per_strut']['z'][i]
                        - (max_tire_diameter / 2 * convert_inch_to_meter), 4),
                    max_section_width * convert_inch_to_meter,
                    max_tire_diameter * convert_inch_to_meter])
                i += 1
            strut_positions.append([landing_gear_description['landing_gear_positions']['nose_gear_strut_y_position'],
                                    landing_gear_description['landing_gear_positions']['nose_gear_z_position']])
            strut_positions.append([landing_gear_description['landing_gear_positions']['nose_gear_strut_y_position'],
                                    landing_gear_description['landing_gear_positions']['nose_gear_z_position']
                                    - landing_gear_description['nose_gear_description']['nose_gear_arrangement'][
                                        'nose_gear_strut_length']])

        # Add main gear tires to tire plot list.
        max_section_width = float(next((value['max_section_width_in_inch'] for value in
                                        landing_gear_description['main_gear_description'][
                                            'main_gear_tires'].values()
                                        if 'max_section_width_in_inch' in value), None))
        max_tire_diameter = float(next((value['max_outside_diameter_in_inch'] for value in
                                        landing_gear_description['main_gear_description'][
                                            'main_gear_tires'].values()
                                        if 'max_outside_diameter_in_inch' in value), None))
        i = 0
        while i < landing_gear_description['main_gear_description'][
                    'main_gear_arrangement']['number_of_main_gear_tires_per_outer_strut'] and i < 2:
            tire_plot_list.append(
                [round(landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position']
                    + landing_gear_description['landing_gear_cog']['main_gear_cog'][
                        'tire_positions_per_outer_strut']['y'][i]
                    - (max_section_width / 2 * convert_inch_to_meter), 4),
                round(landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
                    + landing_gear_description['landing_gear_cog']['main_gear_cog'][
                        'tire_positions_per_outer_strut']['z'][i]
                    - (max_tire_diameter / 2 * convert_inch_to_meter), 4),
                max_section_width * convert_inch_to_meter,
                max_tire_diameter * convert_inch_to_meter])
            i += 1
        strut_positions.append([landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'],
                                landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']])
        strut_positions.append([landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'],
                                landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
                                - landing_gear_description['main_gear_description']['main_gear_arrangement'][
                                    'main_gear_outer_strut_length']])
        strut_positions.append([-landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'],
                                landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']])
        strut_positions.append([-landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'],
                                landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
                                - landing_gear_description['main_gear_description']['main_gear_arrangement'][
                                    'main_gear_outer_strut_length']])

        tire_plot_list.append([-tire_plot_list[-1][0] - tire_plot_list[-1][2],
                            tire_plot_list[-1][1],
                            tire_plot_list[-1][2],
                            tire_plot_list[-1][3]
                            ])
        if landing_gear_description['main_gear_description']['main_gear_arrangement'][
                'number_of_main_gear_tires_per_outer_strut'] > 1:
            tire_plot_list.append([-tire_plot_list[-3][0] - tire_plot_list[-3][2],
                                tire_plot_list[-3][1],
                                tire_plot_list[-3][2],
                                tire_plot_list[-3][3]
                                ])

        if 'number_of_main_gear_inner_struts' in landing_gear_description['main_gear_description']['main_gear_arrangement']:
            i = 0
            while i < landing_gear_description['main_gear_description'][
                    'main_gear_arrangement']['number_of_main_gear_tires_per_inner_strut'] and i < 2:
                tire_plot_list.append(
                    [round(landing_gear_description['landing_gear_positions']['main_gear_inner_strut_y_position']
                        + landing_gear_description['landing_gear_cog']['main_gear_cog'][
                            'tire_positions_per_inner_strut']['y'][i]
                        - (max_section_width / 2 * convert_inch_to_meter), 4),
                    round(landing_gear_description['landing_gear_positions']['main_gear_z_position_inner_strut']
                        + landing_gear_description['landing_gear_cog']['main_gear_cog'][
                            'tire_positions_per_inner_strut']['z'][i]
                        - (max_tire_diameter / 2 * convert_inch_to_meter), 4),
                    max_section_width * convert_inch_to_meter,
                    max_tire_diameter * convert_inch_to_meter])
                i += 1
            strut_positions.append([landing_gear_description['landing_gear_positions']['main_gear_inner_strut_y_position'],
                                    landing_gear_description['landing_gear_positions']['main_gear_z_position_inner_strut']])
            strut_positions.append([landing_gear_description['landing_gear_positions']['main_gear_inner_strut_y_position'],
                                    landing_gear_description['landing_gear_positions']['main_gear_z_position_inner_strut']
                                    - landing_gear_description['main_gear_description']['main_gear_arrangement'][
                                        'main_gear_inner_strut_length']])

            if landing_gear_description['main_gear_description']['main_gear_arrangement'][
                    'number_of_main_gear_inner_struts'] > 1:
                tire_plot_list.append([-tire_plot_list[-1][0] - tire_plot_list[-1][2],
                                    tire_plot_list[-1][1],
                                    tire_plot_list[-1][2],
                                    tire_plot_list[-1][3]
                                    ])
                strut_positions.append(
                    [-landing_gear_description['landing_gear_positions']['main_gear_inner_strut_y_position'],
                    landing_gear_description['landing_gear_positions']['main_gear_z_position_inner_strut']])
                strut_positions.append(
                    [-landing_gear_description['landing_gear_positions']['main_gear_inner_strut_y_position'],
                    landing_gear_description['landing_gear_positions']['main_gear_z_position_inner_strut']
                    - landing_gear_description['main_gear_description']['main_gear_arrangement'][
                        'main_gear_inner_strut_length']])

                if landing_gear_description['main_gear_description']['main_gear_arrangement'][
                        'number_of_main_gear_tires_per_inner_strut'] > 1:
                    tire_plot_list.append([-tire_plot_list[-3][0] - tire_plot_list[-3][2],
                                        tire_plot_list[-3][1],
                                        tire_plot_list[-3][2],
                                        tire_plot_list[-3][3]
                                        ])

        dict_landing_gear_output['tire_plot_list'] = tire_plot_list
        dict_landing_gear_output['strut_positions'] = strut_positions

        dict_landing_gear_output['landing_gear_track'] = \
            [(landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'],
            landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
            - landing_gear_description['main_gear_description']['main_gear_arrangement']['main_gear_outer_strut_length']
            - 2/3 * (max_tire_diameter * convert_inch_to_meter)
            ),
            (-landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'],
            landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
            - landing_gear_description['main_gear_description']['main_gear_arrangement']['main_gear_outer_strut_length']
            - 2/3 * (max_tire_diameter * convert_inch_to_meter)
            ),
            round(abs(2 * landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position']), 2)
            ]

        # Check if wing data are available -> if true: -> prepare plotting.
        _, wing_section_y_position_value = \
            next(iter(dict_ac_exchange['wing_chord_length_on_section'].items()), (None, None))
        if wing_section_y_position_value is not None:
            section_profile_thickness = 0.115
            wing_plot_list = []
            y_pos_list = list(dict_ac_exchange['wing_section_position_in_y'].values())
            z_pos_list = list(dict_ac_exchange['wing_section_position_in_z'].values())
            chord_length = list(dict_ac_exchange['wing_chord_length_on_section'].values())
            i = 0
            while i < len(dict_ac_exchange['wing_chord_length_on_section']):
                section_height = round(chord_length[i] * section_profile_thickness, 4)
                y = round(y_pos_list[i] + dict_ac_exchange['wing_reference_point_global_in_y'], 4)
                z1 = round(z_pos_list[i] + 1/2 * section_height, 4)
                z2 = round(z_pos_list[i] - 1/2 * section_height, 4)
                wing_plot_list.append([y, z1, z2])
                if dict_ac_exchange['wing_symmetry']:
                    wing_plot_list.append([-y, z1, z2])
                i += 1

            dict_landing_gear_output['wing_plot_list'] = sorted(wing_plot_list, key=lambda x: x[0])

            # Generate data for wing clearance plot
            if 'wing_tip_clearance_angles' in landing_gear_description['landing_gear_limitations']:
                wing_clearance_list = [
                    [round(landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position']
                        + abs(landing_gear_description['landing_gear_cog']['main_gear_cog'][
                                    'tire_positions_per_outer_strut']['y'][0])
                        + (max_section_width / 2 * convert_inch_to_meter), 4),
                    round(landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
                        + landing_gear_description['landing_gear_cog']['main_gear_cog'][
                            'tire_positions_per_outer_strut']['z'][0]
                        - (max_tire_diameter / 2 * convert_inch_to_meter), 4)],
                    [y_pos_list[-1],
                    round(landing_gear_description['landing_gear_positions']['main_gear_z_position_outer_strut']
                        + landing_gear_description['landing_gear_cog']['main_gear_cog'][
                            'tire_positions_per_outer_strut']['z'][0]
                        - (max_tire_diameter / 2 * convert_inch_to_meter), 4)],
                    [round(landing_gear_description['landing_gear_limitations']['wing_tip_clearance_angles'], 2)]]

                dict_landing_gear_output['wing_clearance_list'] = wing_clearance_list
            else:
                dict_landing_gear_output['wing_clearance_list'] = None
                
        # Check if engine data are available -> if true: -> prepare plotting.
        _, nacelle_y_position_value = \
            next(iter(dict_ac_exchange['engine_y_position'].items()), (None, None))

        if nacelle_y_position_value is not None:
            nacelle_plot_list = []
            cleaned_nacelle_dict = {key: value for key, value in dict_ac_exchange['nacelle_section_height'].items()
                                    if value is not None}
            i = 0
            y_pos_list = list(dict_ac_exchange['engine_y_position'].values())
            z_pos_list = list(dict_ac_exchange['engine_z_position'].values())
            nacelle_diameter_list = max(list(cleaned_nacelle_dict.values()))
            engine_diameter = nacelle_diameter_list - nacelle_diameter_list / 100 * 9  # 9 percent wall thickness of total nacelle diameter assumed
            nacelle_width = round(nacelle_diameter_list - engine_diameter, 2) * 100  # nacelle wall width in cm for plotting 
            for i in range(0, len(y_pos_list)):
                # List contains the following entries: [y_pos, z_pos, radius, wall_thickness_in_percent]
                nacelle_plot_list.append([y_pos_list[i], z_pos_list[i], nacelle_diameter_list/2, nacelle_width])

            dict_landing_gear_output['nacelle_plot_list'] = nacelle_plot_list

        ''' generate output data for side view of landing gear design '''

        ''' generate output data for top view of landing gear design '''
    
    # Else condition: -> current aircraft configuration is a blended wing body -> plotting currently not implemented
    else:
        dict_landing_gear_output = {}
        runtime_output.warning('Attention! The current aircraft configuration is a blended wing body. \n'
                               '                                    Plotting for this configuration is currently not implemented!')
        
    # debug print
    runtime_output.debug('generate_landing_gear_plots successfully executed!')

    return dict_landing_gear_output
