/*
 * UNICADO - UNIversity Conceptual Aircraft Design and Optimization
 *
 * Copyright (C) 2025 UNICADO consortium
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * Description:
 * This file is part of UNICADO.
 */

#ifndef SYSTEMS_DESIGN_SRC_COMMON_AIRCRAFTSYSTEMS_CONVENTIONALATA29_H_
#define SYSTEMS_DESIGN_SRC_COMMON_AIRCRAFTSYSTEMS_CONVENTIONALATA29_H_

/* The following statements import necessary libraries */
#include <string>                                                   // Import c++ standard library for using string
#include <vector>                                                   // Import c++ standard library for using vector
#include <numeric>                                                  // Import c++ standard library for using numeric
#include <memory>                                                       // Import c++ standard library for using memory

/* The following statements import necessary header files */
#include <svl/SVL.h>                                                // Import of header file from src folder for SVL
#include "../aircraftSystem.h"                                      // Import of header file from src folder for aircraft system
#include "../powerProfile.h"                                        // Import of header file from src folder for power profile
#include "../systemsIOData.h"                                       // Import of header file from systemsIOData for accessing data_. Can't be included in
                                                                    // aircraftSystem.h because that would lead to circular dependencies
#include <aircraftGeometry2/processing/measure.h>

/* Forward declarations */
class fuselage;
class liftingSurface;
class nacelle;
class verticalsurface;

/** \class conventionalATA29
 * \brief class that contains all properties of the hydraulic system (ATA29)
 */
class conventionalATA29: public aircraftSystem {
 public:
    double pumpMass;                            /**<[kg] mass of the pump*/
    double ductingMass;                         /**<[kg] mass of the ducting */
    double fluidMass;                           /**<[kg] mass of the hydraulic fluid */
    double otherMass;                           /**<[kg] mass of the remaining components */

    double lengthDucting;                       /**<[m] length of the ducting */

    double efficiencyFactorHydraulicSystem;/**< [-] efficiency factor of the hydraulic system */
    double maxRelPower;
    double specificDuctingMass; /**< [kg/m] specific mass of hydraulic ducting */
    double specificFluidMass; /**< [kg/m] specific weight of hydraulic fluid */
    double specificPumpMass; /**< [kg/W] power specific weight of hydraulic pumps */

    int numberOfhydraulicCircuits;/**< Number of hydraulic circuits*/

    /**
     * \brief function for reading the system specific parameters from the config
     * 
     * \param config const node&: systemsDesign configuration file
    */
    void readParameters(const node& config) override;
    /**
     * \brief function for updating the system specific values in the aircraft xml
    */
    void updateXML() override;

    /**
     * \brief checks user input in the configuration file for consistency
    */
    void checkUserInput() override;

    /** \brief calculates the power of the conductors
     * \param totalBleed powerProfile
     * \param totalElectric vector<powerProfile>
     * \param totalHydraulic vector<powerProfile>
     * \return void
     */
    void calculateConductingPower(const powerProfile& totalBleed,
                                  const std::vector<powerProfile>& totalElectric,
                                  const std::vector<powerProfile>& totalHydraulic) override;

    /** \brief calculates the conductor mass
     * \param sourceBleed powerProfile
     * \param sourceElectric vector<powerProfile>
     * \param sourceHydraulic vector<powerProfile>
     * \return void
     */
    void calculateConductorMass(const powerProfile& sourceBleed,
                                const std::vector<powerProfile>& sourceElectric,
                                const std::vector<powerProfile>& sourceHydraulic) override;

    /** \brief calculates the power of the hydraulic system (ATA29)
     * \param theHydraulic vector<powerProfile>
     * \return void
     */
    void getPower(std::vector<powerProfile> theHydraulic);

    /** \brief calculates the mass of the hydraulic system (ATA29)
     * \param maxHydPower vector<double>
     * \return double
     */
    double getMass(std::vector<double> maxHydPower);

    /** \brief calculates the reference point of the hydraulic system
     * \return Vec3
     */
    Vec3 getRefPoint();

    /** \brief Constructor of the hydraulic system
     * \param systemName string
     * \param systemIsOperating bool: Switch whether system is operating (1) or not (0)
     * \param theSettings const systemsDesignSettings&
     * \param myAircraft aircraft
     */
    conventionalATA29(const std::string& systemName, bool systemIsOperating, std::shared_ptr<systemsIOData> data, const RuntimeIO& rtIO);
    virtual ~conventionalATA29();

 private:
    /** \brief calculates the length of the ducting
     * \param fuselage Fuselage
     * \param liftingSurface Wing
     * \param vector<geom2::MultisectionSurface<geom2::PolygonSection>> Nacelles
     * \param liftingSurface Stabiliser
     * \param verticalsurface Fin
     * \return double theDuctingLength: [m]
     */
    double getDuctingLength(const geom2::MultisectionSurface<geom2::PolygonSection>& Fuselage, const  std::vector<systemsIOData::Wing> Wing,
                            const std::vector<geom2::MultisectionSurface<geom2::PolygonSection>>& Nacelles,
                            const std::vector<systemsIOData::Empennage> Empennage);
};

#endif // SYSTEMS_DESIGN_SRC_COMMON_AIRCRAFTSYSTEMS_CONVENTIONALATA29_H_
