/*
 * UNICADO - UNIversity Conceptual Aircraft Design and Optimization
 *
 * Copyright (C) 2025 UNICADO consortium
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * Description:
 * This file is part of UNICADO.
 */

#ifndef SYSTEMS_DESIGN_SRC_COMMON_SYSTEMSIODATA_H_
#define SYSTEMS_DESIGN_SRC_COMMON_SYSTEMSIODATA_H_

#include <tuple>
#include <map>
#include <memory>
#include <string>
#include <vector>
#include <aixml/node.h>
#include <aixml/endnode.h>
#include <engine/engine.h>
#include "data.h"
#include "config.h"
#include "missionData.h"
#include "massPropertiesIO.h"

#include <aircraftGeometry2/airfoil_surface.h>
#include <aircraftGeometry2/fuselage.h>
#include <aircraftGeometry2/hull_surface.h>

#include <moduleBasics/runtimeIO.h>

class aircraftSystem;

/**
 * \brief systemsIOData class
*/
class systemsIOData {
 protected:
    const RuntimeIO& rtIO_;

 public:
    std::string aircraftType;/**aircraft type (e.g. CeRAS)*/
    std::string aircraftModel;/** aircraft model and version*/
    std::string aircraftName;/** aircraft name = "type" - "model"*/
    double delta_ISA = 0.;/**< delta temperature to ISA conditions [K] */
    const node& configFile;/** nodes of the configuration file*/

    double calibratedAirspeedUpToTransAltitude; /**< CAS up to transition altitude at FL100 */
    double PreviousOME;/**<[kg] operating mass empty of the previous iteration  */
    double PreviousMME;/**<[kg] manufacturer mass empty of the previous iteration  */

    /** \brief class to define the energy providers for the systems (as defined in the acxml)*/
    class energyProvider {
     public:
        std::string powerTrain;/** power train of the energy provider (e.g. propulsor, turbo, electric, fuel_cell)*/
        std::string missionPhase;/** mission phase in which the energy provider is used*/
        int energyCarrier;/** energy carrier ID (ID's defined in the energy carrier specification node of the acxml)*/
    };
    std::vector<energyProvider> energyProviders;/**vector containing the energy provider for the systems for each mission phase*/

    /** \brief class for the geometric description of a wing and its control surfaces and spars*/
    class Wing {
     public:
        geom2::MultisectionSurface<geom2::AirfoilSection> aerodynamicSurface;/** aerodynamic surface of the wing*/
        Vec3 position;/**[m] Wing root position*/
        std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> controlSurface;/** control surface of the wing*/
        std::vector<double> controlDevicePosDeflection;/**[°] maximum pos. deflection of the control device*/
        std::vector<double> controlDeviceNegDeflection;/**[°] maximum neg. deflection of the control device*/
        double frontSparPos = 0.; /**<[-] relative chord position of front spar (needed for anti icing)*/
    };
    std::vector<Wing> theWing; /** vector containing the geometry information of the wing(s)*/

    /** \brief class for the geometric description of the empennage and its control surfaces*/
    class Empennage {
     public:
        geom2::MultisectionSurface<geom2::AirfoilSection> aerodynamicSurface;/** aerodynamic surface of the empennage*/
        std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> controlSurface;/** control surface of the empennage*/
        std::vector<double> controlDevicePosDeflection;/**[°] maximum pos. deflection of the control device*/
        std::vector<double> controlDeviceNegDeflection;/**[°] maximum neg. deflection of the control device*/
    };
    std::vector<Empennage> theEmpennage;/** vector containing the geometry information of the empennage surfaces*/

    std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> theFuselage;/**vector containing the geometry information of the fuselage(s)*/
    std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> theNacelle;/** vector containing the geometry information of the nacelle(s)*/

    /** \brief class for required landing gear information (name and strut length)*/
    class landing_gear {
     public:
        std::string name;/** landing gear name (e.g. main, nose, ...)*/
        double strut_length;/**<[m] landing gear strut length*/
    };
    std::vector<landing_gear> theLandingGear;/** vecotor containing all landing gears defined in the acxml*/

    /** \brief Class for handling the aircraft systems. The systems are devided into sink, source, virtual, and 
     * conductor systems. And additionally environmental control system, which is seperated to allow for the
     * iteration between the sizing of the ECS and the other systems.
     * \attention The systems are only listed within this class with their name. The detailed definition and sizing
     * of the systems is done in the specific system class (e.g. conventionalATAXX).
    */
    class Systems {
     public:
        std::vector<aircraftSystem*> energySink;/** vector containing all energy sink systems*/
        std::vector<aircraftSystem*> energyECS;/** vector containing all environmental control systems*/
        std::vector<aircraftSystem*> virtualSystem;/** vector containing all virtual systems*/
        std::vector<aircraftSystem*> energySource;/** vector containing all energy source systems*/
        std::vector<aircraftSystem*> energyConductor;/** vector containing all energy conducting systems*/
        std::vector<aircraftSystem*> allSystems;/** vector containing all systems*/

        /**
         * \brief puts all systems (energy sinks, conductors, sources, and virtual systems) into one vector allSystems
         * 
        */
        void getAllSystems() {
            for (size_t i = 0; i < energySink.size(); i++) {
                allSystems.push_back(energySink.at(i));
            }
            for (size_t i = 0; i < energyConductor.size(); i++) {
                allSystems.push_back(energyConductor.at(i));
            }
            for (size_t i = 0; i < energySource.size(); i++) {
                allSystems.push_back(energySource.at(i));
            }
            for (size_t i = 0; i < virtualSystem.size(); i++) {
                allSystems.push_back(virtualSystem.at(i));
            }
        }

        /**
         * \brief returns the ID of a specific system within the allSystems vector
         * 
         * \param ATA ATA chapter of the system
         * 
         * \returns ID of the specific system
        */
        int getSpecificSystem(const std::string& ATA) {
            bool systemFound(false);
            int ID(0);
            for (size_t i = 0; i < allSystems.size(); i++) {
                if (allSystems.at(i)->ATA == ATA) {
                    ID = i;
                    systemFound = true;
                    break;
                }
            }
            if (systemFound == false) {
                myRuntimeInfo->err << "The system could not be found.: " << ATA << std::endl;
                myRuntimeInfo->err << "Abort in systemsIOData.h, getSpecificSystem!" << std::endl;
                exit(1);
            }
            return ID;
        }
        /**
         * \brief returns the ID of a specific system within the allSystems vector
         * 
         * \param ATA ATA chapter of the system
         * 
         * \returns bool
        */
        bool systemExists(const std::string& ATA) {
            bool systemFound(false);
            for (size_t i = 0; i < allSystems.size(); i++) {
                if (allSystems.at(i)->ATA == ATA) {
                    systemFound = true;
                }
            }
            return systemFound;
        }
        /**
         * \brief checks if system is contained in all systems
         * 
         * \param ATA const std::string& (ATA chapter of the system)
         * 
         * \return bool systemFound
        */
        bool containsSystem(const std::string& ATA) {
            bool systemFound(false);
            for (size_t i = 0; i < allSystems.size(); i++) {
                if (allSystems.at(i)->ATA == ATA) {
                    systemFound = true;
                    return systemFound;
                }
            } return systemFound;
        }
    };
    Systems Systems;/**contains all systems*/
    standardData data;/**contains information from the acxml*/
    standardConfig configuration;/**contains information from the config*/
    missionData mission;/**contains mission information*/

    /**
     * \brief construct a new systems IO data object
     * 
     * \param rtIO rtIO pointer
     * \param config Node of configuration xml
     * \param mission_data_dir mission data directory
    */
    systemsIOData(const RuntimeIO& rtIO, const node& config, std::string mission_data_dir);

    /**
     * \brief get mission file depending on reqMission
     * \param mission_data_dir mission data directory
     * \return missionFile
    */
    std::string getMissionFile(std::string mission_data_dir);

    /**
     * \brief Reads all wing surface geometry and the wings control surfaces and spars from
     * the acxml.
     * \param acxml aircraft xml
     * \param airfoilDir airfoil directory
     * \return void
     */
    void readWing(const node& acxml, const std::string& airfoilDir);

    /**
     * \brief Reads all empennage surfaces and their control devices from the acxml.
     * \note current implementation checks for fin and horizontal tail since aircraft systems
     * do not yet have methods for v-tail or other unconventional tails.
     * \param acxml aircraft xml
     * \param airfoilDir airfoil directory
     * \return void
     */
    void readEmpennage(const node& acxml, const std::string& airfoilDir);

    /**
     * \brief Reads all fuselages from the acxml.
     * \param acxml aircraft xml
     * \param geometryDir geometry directory (needed for geometry files)
     * \return void
     */
    void readFuselages(const node& acxml, const std::string& geometryDir);

    /**
     * \brief Reads all nacelles from acxml (for all propulsors)
     * \param acxml aircraft xml
     * \param geometryDir geometry directory (needed for geometry files)
     * \return void
     */
    void readNacelles(const node& acxml, const std::string& geometryDir);

    /**
     * \brief Reads strut length and landing gear name of each gear from the acxml.
     * \param geometry Geometry elements node of aircraft xml
     */
    void readLandingGears(const node& geometry);

    /**
     * \brief Reads mass and CG components from acxml
     * \param acxml aircraft xml
     * \return void
    */
    void readMassesFromXML(const node& acxml) {
        readMassComponents(acxml);
        readCGComponents(acxml);
        PreviousOME = data.Mass.OME;
    }

    /**
     * \brief Reads the required mass components from the acxml.
     * \param acxml aircraft xml
     * \todo check if each of the values is actually used.
     * \todo read value for fuel density from acxml
     * \return void
    */
    void readMassComponents(const node& acxml);

    /**
     * \brief Reads the required center of gravity position of components from the acxml.
     * \param acxml aircraft xml
     * \return void
    */
    void readCGComponents(const node& acxml);

    /**
     * \brief Reads performance data from the acxml. 
     * \param acxml aircraft xml
     * \return void
    */
    void readPerformance(const node& acxml);

    /**
     * \brief Loops through all fuselages, payload tubes, and payload decks to read the
     * accommodation data. 
     * \param acxml aircraft xml
     * \todo read value for flight crew number from acxml
     * \return void
    */
    void readAccomodation(const node& acxml);

    /**
     * \brief Loops through all propulsor to reads the propulsion data from the acxml and enginexml.
     * \param acxml aircraft xml
     * \param engine_dir path to engine directory
     * \todo value from engine xml potentially from acxml?
     * \return void
    */
    void readPropulsion(const node& acxml, std::filesystem::path engine_dir);

    /**
     * \brief Updates the mission xml file with the power requirement of the systems for each mission step.
     * \param mission_data_dir mission data directory
     * \note electric and hydraulic power are converted to shaft offtakes from the engine. Bleed air becomes
     * bleed offtakes from the engine. Currently, the dummy system ATA-70 (engine) is used to sum up the total
     * power required by the systems and thus the ATA-70 power profile is used here.
    */
    void updateMissionXML(std::string mission_data_dir);

    /** \brief Update function for the component mass properties of systems in the acxml.
     * \param acxml aircraft xml
     * \return void
    */
    void updateMassProperties();

    /** \brief returns the postion of a component in the acxml
     *  \param acxml aircraft xml
     *  \param component_path path to the component in the acxml
     *  \return Vec3 CofG returns the components positon
    */
    Vec3 readPosition(const node& acxml, std::string component_path);

    /** \brief Checks if value is outside (< or >) of boundaries
     * \param name name to identify the value
     * \param value value to be checked
     * \param minValue lower boundary
     * \param maxValue upper boundary
     * 
     * \return void
    */
    void checkBoundaries(std::string name, double value, const double& minValue, const double& maxValue);

    /** \brief sets all values to zero of an accumulatedPowerConsumption
     *  \param theConsumption pointer to the consumption that shall be cleared
     * \return void
    */
    void clearInitializePowerConsumption(standardData::accumulatedPowerConsumption *theConsumption);

    /** \brief function for updating the mission xml in the virtual system strategy.
     * \param mission_data_dir mission data directory
     * \note electric and hydraulic power are converted to shaft offtakes from the engine. Bleed air becomes
     * bleed offtakes from the engine.
     */
    void update_mission_xml_virtual(std::string mission_data_dir);
};

#endif // SYSTEMS_DESIGN_SRC_COMMON_SYSTEMSIODATA_H_
